package org.eclipse.jdt.internal.compiler.apt.util;

import java.net.URISyntaxException;
import java.net.URI;
import java.io.Writer;
import java.io.Reader;
import java.io.OutputStream;
import java.io.InputStream;
import java.util.zip.ZipEntry;
import org.eclipse.jdt.internal.compiler.util.Util;
import javax.lang.model.element.NestingKind;
import org.eclipse.jdt.internal.compiler.classfmt.ClassFormatException;
import org.eclipse.jdt.internal.compiler.classfmt.ClassFileReader;
import javax.lang.model.element.Modifier;
import java.io.IOException;
import java.nio.charset.Charset;
import java.util.zip.ZipFile;
import java.io.File;
import javax.tools.JavaFileObject;

public class ArchiveFileObject implements JavaFileObject
{
    private String entryName;
    private File file;
    private ZipFile zipFile;
    private Charset charset;
    private static /* synthetic */ int[] $SWITCH_TABLE$javax$tools$JavaFileObject$Kind;
    
    public ArchiveFileObject(File file, String entryName, Charset charset) {
        this.entryName = entryName;
        this.file = file;
        this.charset = charset;
    }
    
    @Override
    protected void finalize() throws Throwable {
        if (this.zipFile != null) {
            try {
                this.zipFile.close();
            }
            catch (IOException ex) {}
        }
        super.finalize();
    }
    
    @Override
    public Modifier getAccessLevel() {
        if (this.getKind() != JavaFileObject.Kind.CLASS) {
            return null;
        }
        ClassFileReader reader = null;
        try {
            Throwable t = null;
            try {
                ZipFile zip = new ZipFile(this.file);
                try {
                    reader = ClassFileReader.read(zip, this.entryName);
                }
                finally {
                    if (zip != null) {
                        zip.close();
                    }
                }
            }
            finally {
                if (t == null) {
                    Throwable exception;
                    t = exception;
                }
                else {
                    Throwable exception;
                    if (t != exception) {
                        t.addSuppressed(exception);
                    }
                }
            }
        }
        catch (ClassFormatException ex) {}
        catch (IOException ex2) {}
        if (reader == null) {
            return null;
        }
        int accessFlags = reader.accessFlags();
        if ((accessFlags & 0x1) != 0x0) {
            return Modifier.PUBLIC;
        }
        if ((accessFlags & 0x400) != 0x0) {
            return Modifier.ABSTRACT;
        }
        if ((accessFlags & 0x10) != 0x0) {
            return Modifier.FINAL;
        }
        return null;
    }
    
    @Override
    public JavaFileObject.Kind getKind() {
        String name = this.entryName.toLowerCase();
        if (name.endsWith(JavaFileObject.Kind.CLASS.extension)) {
            return JavaFileObject.Kind.CLASS;
        }
        if (name.endsWith(JavaFileObject.Kind.SOURCE.extension)) {
            return JavaFileObject.Kind.SOURCE;
        }
        if (name.endsWith(JavaFileObject.Kind.HTML.extension)) {
            return JavaFileObject.Kind.HTML;
        }
        return JavaFileObject.Kind.OTHER;
    }
    
    @Override
    public NestingKind getNestingKind() {
        switch ($SWITCH_TABLE$javax$tools$JavaFileObject$Kind()[this.getKind().ordinal()]) {
            case 1: {
                return NestingKind.TOP_LEVEL;
            }
            case 2: {
                ClassFileReader reader = null;
                try {
                    Throwable t = null;
                    try {
                        ZipFile zip = new ZipFile(this.file);
                        try {
                            reader = ClassFileReader.read(zip, this.entryName);
                        }
                        finally {
                            if (zip != null) {
                                zip.close();
                            }
                        }
                    }
                    finally {
                        if (t == null) {
                            Throwable exception;
                            t = exception;
                        }
                        else {
                            Throwable exception;
                            if (t != exception) {
                                t.addSuppressed(exception);
                            }
                        }
                    }
                }
                catch (ClassFormatException ex) {}
                catch (IOException ex2) {}
                if (reader == null) {
                    return null;
                }
                if (reader.isAnonymous()) {
                    return NestingKind.ANONYMOUS;
                }
                if (reader.isLocal()) {
                    return NestingKind.LOCAL;
                }
                if (reader.isMember()) {
                    return NestingKind.MEMBER;
                }
                return NestingKind.TOP_LEVEL;
            }
            default: {
                return null;
            }
        }
    }
    
    @Override
    public boolean isNameCompatible(String simpleName, JavaFileObject.Kind kind) {
        return this.entryName.endsWith(simpleName + kind.extension);
    }
    
    @Override
    public boolean delete() {
        throw new UnsupportedOperationException();
    }
    
    @Override
    public boolean equals(Object o) {
        if (!(o instanceof ArchiveFileObject)) {
            return false;
        }
        ArchiveFileObject archiveFileObject = (ArchiveFileObject)o;
        return archiveFileObject.toUri().equals(this.toUri());
    }
    
    @Override
    public int hashCode() {
        return this.toUri().hashCode();
    }
    
    @Override
    public CharSequence getCharContent(boolean ignoreEncodingErrors) throws IOException {
        if (this.getKind() == JavaFileObject.Kind.SOURCE) {
            Throwable t = null;
            try {
                ZipFile zipFile2 = new ZipFile(this.file);
                try {
                    ZipEntry zipEntry = zipFile2.getEntry(this.entryName);
                    CharSequence charContents = org.eclipse.jdt.internal.compiler.apt.util.Util.getCharContents(this, ignoreEncodingErrors, Util.getZipEntryByteContent(zipEntry, zipFile2), this.charset.name());
                }
                finally {
                    if (zipFile2 != null) {
                        zipFile2.close();
                    }
                }
            }
            finally {
                if (t == null) {
                    Throwable exception;
                    t = exception;
                }
                else {
                    Throwable exception;
                    if (t != exception) {
                        t.addSuppressed(exception);
                    }
                }
            }
        }
        return null;
    }
    
    @Override
    public long getLastModified() {
        try {
            Throwable t = null;
            try {
                ZipFile zip = new ZipFile(this.file);
                try {
                    ZipEntry zipEntry = zip.getEntry(this.entryName);
                    return zipEntry.getTime();
                }
                finally {
                    if (zip != null) {
                        zip.close();
                    }
                }
            }
            finally {
                if (t == null) {
                    Throwable exception;
                    t = exception;
                }
                else {
                    Throwable exception;
                    if (t != exception) {
                        t.addSuppressed(exception);
                    }
                }
            }
        }
        catch (IOException ex) {
            return 0L;
        }
    }
    
    @Override
    public String getName() {
        return this.entryName;
    }
    
    @Override
    public InputStream openInputStream() throws IOException {
        if (this.zipFile == null) {
            this.zipFile = new ZipFile(this.file);
        }
        ZipEntry zipEntry = this.zipFile.getEntry(this.entryName);
        return this.zipFile.getInputStream(zipEntry);
    }
    
    @Override
    public OutputStream openOutputStream() throws IOException {
        throw new UnsupportedOperationException();
    }
    
    @Override
    public Reader openReader(boolean ignoreEncodingErrors) throws IOException {
        throw new UnsupportedOperationException();
    }
    
    @Override
    public Writer openWriter() throws IOException {
        throw new UnsupportedOperationException();
    }
    
    @Override
    public URI toUri() {
        try {
            return new URI("jar:" + this.file.toURI().getPath() + "!" + this.entryName);
        }
        catch (URISyntaxException ex) {
            return null;
        }
    }
    
    @Override
    public String toString() {
        return this.file.getAbsolutePath() + "[" + this.entryName + "]";
    }
    
    static /* synthetic */ int[] $SWITCH_TABLE$javax$tools$JavaFileObject$Kind() {
        int[] $switch_TABLE$javax$tools$JavaFileObject$Kind = ArchiveFileObject.$SWITCH_TABLE$javax$tools$JavaFileObject$Kind;
        if ($switch_TABLE$javax$tools$JavaFileObject$Kind != null) {
            return $switch_TABLE$javax$tools$JavaFileObject$Kind;
        }
        int[] $switch_TABLE$javax$tools$JavaFileObject$Kind2 = new int[JavaFileObject.Kind.values().length];
        try {
            $switch_TABLE$javax$tools$JavaFileObject$Kind2[JavaFileObject.Kind.CLASS.ordinal()] = 2;
        }
        catch (NoSuchFieldError noSuchFieldError) {}
        try {
            $switch_TABLE$javax$tools$JavaFileObject$Kind2[JavaFileObject.Kind.HTML.ordinal()] = 3;
        }
        catch (NoSuchFieldError noSuchFieldError2) {}
        try {
            $switch_TABLE$javax$tools$JavaFileObject$Kind2[JavaFileObject.Kind.OTHER.ordinal()] = 4;
        }
        catch (NoSuchFieldError noSuchFieldError3) {}
        try {
            $switch_TABLE$javax$tools$JavaFileObject$Kind2[JavaFileObject.Kind.SOURCE.ordinal()] = 1;
        }
        catch (NoSuchFieldError noSuchFieldError4) {}
        return ArchiveFileObject.$SWITCH_TABLE$javax$tools$JavaFileObject$Kind = $switch_TABLE$javax$tools$JavaFileObject$Kind2;
    }
}
